using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace BendayObjectDataSource.Common
{
    public class PropertyReflector
    {
        private readonly object[] m_emptyArrayOfObject = new object[0] { };

        public PropertyReflector(Type reflectedType, string propertyName)
        {
            if (reflectedType == null)
            {
                throw new ArgumentNullException("reflectedType", "Argument cannot be null.");
            }
            else if (propertyName == null)
            {
                throw new ArgumentNullException("propertyName", "Argument cannot be null.");
            }

            var property = reflectedType.GetProperty(propertyName);

            if (property == null)
            {
                throw new ArgumentException("Unknown property name on type.", "propertyName");
            }

            Property = property;
        }

        private PropertyInfo Property { get; set; }



        public string GetValueAsString(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance", "Argument cannot be null.");
            }

            object temp = Property.GetValue(instance, null);

            if (temp == null)
            {
                return null;
            }
            else
            {
                return temp.ToString();
            }
        }

        public Type PropertyType
        {
            get
            {
                if (Property == null)
                {
                    return null;
                }
                else
                {
                    return Property.PropertyType;
                }
            }
        }

        public DateTime GetValueAsDateTime(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance", "Argument cannot be null.");
            }

            object temp = Property.GetValue(instance, null);

            if (temp == null)
            {
                return DateTime.MinValue;
            }
            else
            {
                return Convert.ToDateTime(temp);
            }
        }

        public int GetValueAsInt32(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance", "Argument cannot be null.");
            }

            object temp = Property.GetValue(instance, null);

            if (temp == null)
            {
                return 0;
            }
            else
            {
                return Convert.ToInt32(temp);
            }
        }

        public float GetValueAsFloat(object instance)
        {
            if (instance == null)
            {
                throw new ArgumentNullException("instance", "Argument cannot be null.");
            }

            object temp = Property.GetValue(instance, null);

            if (temp == null)
            {
                return 0;
            }
            else
            {
                return Convert.ToSingle(temp);
            }
        }

        private void SetValueAsObject(object instance, object value)
        {
            Property.SetValue(instance, value, m_emptyArrayOfObject);
        }

        private static float ParseFloat(string value)
        {
            float returnValue = 0f;

            if (float.TryParse(value, out returnValue) == true)
            {
                
            }

            return returnValue;
        }

        private static int ParseInt32(string value)
        {
            int returnValue = 0;

            if (int.TryParse(value, out returnValue) == true)
            {

            }

            return returnValue;
        }

        private static DateTime ParseDateTime(string value)
        {
            DateTime returnValue = new DateTime();

            if (DateTime.TryParse(value, out returnValue) == true)
            {

            }

            return returnValue;
        }

        public void SetValue(object instance, string value)
        {
            if (Property.PropertyType == typeof(string))
            {
                SetValueAsObject(instance, value);
            }
            else if (Property.PropertyType == typeof(DateTime))
            {
                SetValueAsObject(instance, ParseDateTime(value));
            }
            else if (Property.PropertyType == typeof(float))
            {
                SetValueAsObject(instance, ParseFloat(value));
            }
            else if (Property.PropertyType == typeof(int))
            {
                SetValueAsObject(instance, ParseInt32(value));
            }
        }
    }
}
