using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using BendayObjectDataSource.FakeBusinessTier;
using BendayObjectDataSource.Common;

namespace BendayObjectDataSource.UnitTests
{
    
    [TestClass]
    public class ReflectionFixture
    {
        public ReflectionFixture()
        {
        }

        [TestMethod]
        public void GetStringValueByReflection()
        {
            var persons = new PersonFacade().GetAllPersons();

            AssertGetStringValueByReflection(persons[0], "EmailAddress", persons[0].EmailAddress);
            AssertGetStringValueByReflection(persons[0], "FirstName", persons[0].FirstName);
            AssertGetStringValueByReflection(persons[0], "LastName", persons[0].LastName);
            AssertGetStringValueByReflection(persons[0], "Status", persons[0].Status);
        }

        private static void AssertGetStringValueByReflection(Person person, string propertyName, string expected)
        {
            Assert.IsFalse(String.IsNullOrEmpty(expected), "Expected value was null or empty.");

            PropertyReflector reflector = new PropertyReflector(typeof(Person), propertyName);

            Assert.AreEqual<string>(expected,
                reflector.GetValueAsString(person),
                "GetAsString returned wrong value.");
        }

        [TestMethod]
        public void GetIntValueByReflection()
        {
            var person = new PersonFacade().GetAllPersons()[0];

            PropertyReflector reflector = new PropertyReflector(typeof(Person), "Id");

            Assert.AreEqual<int>(person.Id, reflector.GetValueAsInt32(person), "Wrong value.");
        }

        [TestMethod]
        public void GetPropertyDataType()
        {
            PropertyReflector reflector;
            
            reflector = new PropertyReflector(typeof(Person), "Id");
            Assert.AreEqual<Type>(typeof(int), reflector.PropertyType, "PropertyType was wrong.");

            reflector = new PropertyReflector(typeof(Person), "FirstName");
            Assert.AreEqual<Type>(typeof(string), reflector.PropertyType, "PropertyType was wrong.");
            
            reflector = new PropertyReflector(typeof(Person), "Salary");
            Assert.AreEqual<Type>(typeof(float), reflector.PropertyType, "PropertyType was wrong.");
        }
        
        
        
    }
}
