﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using BendayObjectDataSource.Common;
using BendayObjectDataSource.FakeBusinessTier;

namespace BendayObjectDataSource.UnitTests
{

    [TestClass]
    public class SortingFixture
    {
        public SortingFixture()
        {
        }

        private static void ParseAndAssert(string sortByString, string expectedSortBy, SortDirection expectedDirection)
        {
            SortArgument arg = DataSourceUtility.ParseSortArguments(sortByString);

            Assert.IsNotNull(arg, "Argument was null.");
            Assert.AreEqual<string>(expectedSortBy, arg.SortBy, "SortBy was wrong.");
            Assert.AreEqual<SortDirection>(expectedDirection, arg.Direction, "Direction was wrong.");
        }

        [TestMethod]
        public void ParseSortArguments()
        {
            ParseAndAssert("firstname desc", "firstname", SortDirection.Descending);
            ParseAndAssert("firstname asc", "firstname", SortDirection.Ascending);
            ParseAndAssert("firstname", "firstname", SortDirection.Ascending);

            ParseAndAssert("LastName desc", "LastName", SortDirection.Descending);
            ParseAndAssert("LastName asc", "LastName", SortDirection.Ascending);
            ParseAndAssert("LastName", "LastName", SortDirection.Ascending);

            ParseAndAssert("LastName Desc", "LastName", SortDirection.Descending);
            ParseAndAssert("LastName Asc", "LastName", SortDirection.Ascending);

            ParseAndAssert("LastName DESC", "LastName", SortDirection.Descending);
            ParseAndAssert("LastName ASC", "LastName", SortDirection.Ascending);

            ParseAndAssert(" LastName DESC ", "LastName", SortDirection.Descending);
            ParseAndAssert(" LastName ASC ", "LastName", SortDirection.Ascending);
            ParseAndAssert(" LastName ", "LastName", SortDirection.Ascending);
        }

        [TestMethod]
        public void ParseThrowsExceptionOnContainsSpaceInSortByName()
        {
            try
            {
                DataSourceUtility.ParseSortArguments("Last Name");
                Assert.Fail();
            }
            catch (Exception)
            {
                
            }

            try
            {
                DataSourceUtility.ParseSortArguments("Last Name DESC");
                Assert.Fail();
            }
            catch (Exception)
            {

            }

            try
            {
                DataSourceUtility.ParseSortArguments("Last Name ASC");
                Assert.Fail();
            }
            catch (Exception)
            {

            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionAscending()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.FirstName));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("FirstName", SortDirection.Ascending));

            string previousValue = null;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.FirstName));
                Assert.IsTrue(String.Compare(previousValue, item.FirstName) < 0, "Previous value was not less than current.");
                previousValue = item.FirstName;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionDesc()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.FirstName));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("FirstName", SortDirection.Descending));

            string previousValue = "zzzzzzzzzzzzzzzzzzzzzz";

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.FirstName));
                Assert.IsTrue(String.Compare(previousValue, item.FirstName) > 0, "Previous value was not less than current.");
                previousValue = item.FirstName;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionAscendingByInt32()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.Id));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("Id", SortDirection.Ascending));

            int previousValue = -1;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.Id));
                Assert.IsTrue(previousValue < item.Id, "Previous value was not less than current.");
                previousValue = item.Id;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionDescendingByInt32()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.Id));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("Id", SortDirection.Descending));

            int previousValue = Int32.MaxValue;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.Id));
                Assert.IsTrue(previousValue >= item.Id, "Previous value was not less than current.");
                previousValue = item.Id;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionAscendingByDouble()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.Salary));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("Salary", SortDirection.Ascending));

            float previousValue = -1f;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.Salary));
                Assert.IsTrue(previousValue <= item.Salary, "Previous value was not less than current.");
                previousValue = item.Salary;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionDescendingByDouble()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.Salary));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("Salary", SortDirection.Descending));

            float previousValue = float.MaxValue;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.Salary));
                Assert.IsTrue(previousValue >= item.Salary, "Previous value was not less than current.");
                previousValue = item.Salary;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionAscendingByDateTime()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.CreatedDate));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("CreatedDate", SortDirection.Ascending));

            DateTime previousValue = DateTime.MinValue;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.CreatedDate));
                Assert.IsTrue(previousValue <= item.CreatedDate, "Previous value was not less than current.");
                previousValue = item.CreatedDate;
            }
        }

        [TestMethod]
        public void SortCollectionUsingReflectionDescendingByDateTime()
        {
            var unsorted = new PersonFacade().GetAllPersons();

            foreach (var item in unsorted)
            {
                Console.WriteLine(String.Format("Unsorted: {0}", item.CreatedDate));
            }

            var sorted = DataSourceUtility.Sort(unsorted, new SortArgument("CreatedDate", SortDirection.Descending));

            DateTime previousValue = DateTime.MaxValue;

            foreach (var item in sorted)
            {
                Console.WriteLine(String.Format("Sorted: {0}", item.CreatedDate));
                Assert.IsTrue(previousValue >= item.CreatedDate, "Previous value was not greater than current.");
                previousValue = item.CreatedDate;
            }
        }

        [TestMethod]
        public void PadFloat()
        {
            int expectedMinimumStringLength = float.MaxValue.ToString().Length;

            Assert.AreEqual<int>(12, expectedMinimumStringLength, "The expected minimum string length assumption was wrong.");

            Assert.AreEqual<string>("0000000000000000000.982339978218079", DataSourceUtility.PadFloat(0.98234f), "0.98234f wasn't formatted properly.");
            Assert.AreEqual<string>("0000000000000000001.0", DataSourceUtility.PadFloat(1f), "1f wasn't formatted properly.");
            Assert.AreEqual<string>("0000000000000000001.810999989509583", DataSourceUtility.PadFloat(1.811f), "1.811f wasn't formatted properly.");
        }

        [TestMethod]
        public void PadInt32()
        {
            int expectedStringLength = Int32.MaxValue.ToString().Length;

            Assert.AreEqual<int>(10, expectedStringLength, "The expected string length assumption was wrong.");

            Assert.AreEqual<string>("0000000000", DataSourceUtility.PadInt32(0), "0 wasn't formatted properly.");
            Assert.AreEqual<string>("0000000010", DataSourceUtility.PadInt32(10), "10 wasn't formatted properly.");
            Assert.AreEqual<string>(Int32.MaxValue.ToString(), DataSourceUtility.PadInt32(Int32.MaxValue), "MaxValue wasn't formatted properly.");
        }

        [TestMethod]
        public void PadDateTime()
        {
            int expectedStringLength = DateTime.MaxValue.Ticks.ToString().Length;

            Assert.AreEqual<int>(19, expectedStringLength, "The expected string length assumption was wrong.");

            Assert.AreEqual<string>("0633504672000000000.0", DataSourceUtility.PadDateTime(new DateTime(2008, 7, 1)), "7/1/2008 wasn't formatted properly.");
        }
        
    }
}
